#include "ViewCellsTree.h"
#include "Timer/PerfTimer.h"
#include <stack>
#include <string>

using namespace std;


// MAGIC of all bin exports
#ifndef MAGIC 
#define MAGIC 0x827923
#endif

#define VIEWCELLS_VERSION 1.0f

namespace CHCDemoEngine
{


ViewCell *ViewCellsTree::GetViewCell(const Vector3 &point) const
{
	ViewCellsTreeNode *node = mRoot;

	while (!node->IsLeaf()) 
	{
		if (point[node->mAxis] < node->mPosition)
			node = node->mBack;
		else
			node = node->mFront;
	}

	return node->mViewCell;
}


bool ViewCellsTree::LoadFromFile(const std::string &filename)
{
	cout << "Info: Loading view cells from file '" + filename + "'";

	FILE *fr = fopen(filename.c_str(), "rb");

	if (!fr) 
	{
		cerr << "Error: Cannot open file for reading" << endl;
		return false;
	}

	bool result = _LoadFromFile(fr);

	cout << "Info: done. " << endl;
	fclose(fr);

	return result;
}


bool ViewCellsTree::_LoadFromFile(FILE *fr)
{
	int buffer[256]; 
	fread(buffer, sizeof(int), 3, fr);

	if (buffer[0] != MAGIC) 
	{
		cerr << "Error: Wrong file type" << endl;
		return false;
	}

	if (buffer[1] != (int)(1000 * VIEWCELLS_VERSION)) 
	{
		cerr << "Error: Wrong viewcells version" << endl;
		return false;
	}

	// get the bounding box
	fread(&mBox, sizeof(AxisAlignedBox3), 1, fr);

	stack<ViewCellsTreeNode **> nodeStack;

	nodeStack.push(&mRoot);

	while (!nodeStack.empty()) 
	{
		ViewCellsTreeNode *&node = *nodeStack.top();
		nodeStack.pop();
		node = new ViewCellsTreeNode();

		fread(&node->mAxis, sizeof(int), 1, fr);

		if (!node->IsLeaf())
		{
			fread(&node->mPosition, sizeof(float), 1, fr);
			nodeStack.push(&node->mFront);
			nodeStack.push(&node->mBack);
		}
	}

	return true;
}


}